@@ -90,7 +90,7 @@ public class InlineDetachCmdTest extends CLITest {
         pipeStringToStdin(msgOut.toString());
         ByteArrayOutputStream verifyOut = pipeStdoutToStream();
         assertSuccess(executeCommand("verify", sigFile.getAbsolutePath(), certFile.getAbsolutePath()));
-        assertEquals("2021-05-15T16:08:06Z 4F665C4DC2C4660BC6425E415736E6931ACF370C 4F665C4DC2C4660BC6425E415736E6931ACF370C\n",
+        assertEquals("2021-05-15T16:08:06Z 4F665C4DC2C4660BC6425E415736E6931ACF370C 4F665C4DC2C4660BC6425E415736E6931ACF370C mode:text\n",
                 verifyOut.toString());
     }
 
@@ -115,7 +115,7 @@ public class InlineDetachCmdTest extends CLITest {
         ByteArrayOutputStream verifyOut = pipeStdoutToStream();
         File certFile = writeFile("cert.asc", CERT);
         assertSuccess(executeCommand("verify", sigFile.getAbsolutePath(), certFile.getAbsolutePath()));
-        assertEquals("2021-05-15T16:08:06Z 4F665C4DC2C4660BC6425E415736E6931ACF370C 4F665C4DC2C4660BC6425E415736E6931ACF370C\n",
+        assertEquals("2021-05-15T16:08:06Z 4F665C4DC2C4660BC6425E415736E6931ACF370C 4F665C4DC2C4660BC6425E415736E6931ACF370C mode:text\n",
                 verifyOut.toString());
     }
 
@@ -129,4 +129,27 @@ public class InlineDetachCmdTest extends CLITest {
         assertEquals(0, msgOut.size());
     }
 
+    @Test
+    public void detachNonOpenPgpDataFails() throws IOException {
+        File sig = nonExistentFile("sig.asc");
+        pipeStringToStdin("This is non-OpenPGP data and therefore we cannot detach any signatures from it.");
+        int exitCode = executeCommand("inline-detach", "--signatures-out", sig.getAbsolutePath());
+
+        assertEquals(SOPGPException.BadData.EXIT_CODE, exitCode);
+    }
+
+    @Test
+    public void detachMissingSignaturesFromCleartextSignedMessageFails() throws IOException {
+        String cleartextSignedNoSigs = "-----BEGIN PGP SIGNED MESSAGE-----\n" +
+                "\n" +
+                "Hello, World!\n" +
+                "What's Up!??\n" +
+                "\n" +
+                "\n";
+        pipeStringToStdin(cleartextSignedNoSigs);
+        File sig = nonExistentFile("sig.asc");
+        int exitCode = executeCommand("inline-detach", "--signatures-out", sig.getAbsolutePath());
+
+        assertEquals(SOPGPException.BadData.EXIT_CODE, exitCode);
+    }
 }
