@@ -18,6 +18,7 @@ import com.ginsberg.junit.exit.FailOnSystemExit;
 import org.bouncycastle.openpgp.PGPException;
 import org.bouncycastle.openpgp.PGPPublicKeyRing;
 import org.bouncycastle.openpgp.PGPSecretKeyRing;
+import org.junit.jupiter.api.Disabled;
 import org.junit.jupiter.api.Test;
 import org.pgpainless.PGPainless;
 import org.pgpainless.algorithm.KeyFlag;
@@ -129,7 +130,7 @@ public class RoundTripEncryptDecryptCmdTest extends CLITest {
         String romeosVerif = readStringFromFile(anotherVerificationsFile);
         assertEquals(julietsVerif, romeosVerif);
         assertFalse(julietsVerif.isEmpty());
-        assertEquals(103, julietsVerif.length()); // 103 is number of symbols in [DATE, FINGER, FINGER] for V4
+        assertEquals(115, julietsVerif.length()); // 115 is number of symbols in [DATE, FINGER, FINGER, MODE] for V4
     }
 
     @Test
@@ -139,6 +140,7 @@ public class RoundTripEncryptDecryptCmdTest extends CLITest {
     }
 
     @Test
+    @Disabled("Disabled, since we now read certificates from secret keys")
     public void testEncryptingForKeyFails() throws IOException {
         File notACert = writeFile("key.asc", KEY);
 
@@ -168,7 +170,8 @@ public class RoundTripEncryptDecryptCmdTest extends CLITest {
 
         File verifications = nonExistentFile("verifications");
         ByteArrayOutputStream out = pipeStdoutToStream();
-        int exitCode = executeCommand("decrypt", "--verifications-out", verifications.getAbsolutePath(), key.getAbsolutePath());
+        int exitCode = executeCommand("decrypt", "--verifications-out",
+                verifications.getAbsolutePath(), key.getAbsolutePath());
 
         assertEquals(SOPGPException.IncompleteVerification.EXIT_CODE, exitCode);
         assertEquals(0, out.size());
@@ -211,7 +214,8 @@ public class RoundTripEncryptDecryptCmdTest extends CLITest {
 
         pipeStringToStdin(ciphertext);
         ByteArrayOutputStream plaintextOut = pipeStdoutToStream();
-        assertSuccess(executeCommand("decrypt", "--session-key-out", sessionKeyFile.getAbsolutePath(), key.getAbsolutePath()));
+        assertSuccess(executeCommand("decrypt", "--session-key-out",
+                sessionKeyFile.getAbsolutePath(), key.getAbsolutePath()));
 
         assertEquals(plaintext, plaintextOut.toString());
         String resultSessionKey = readStringFromFile(sessionKeyFile);
@@ -272,7 +276,7 @@ public class RoundTripEncryptDecryptCmdTest extends CLITest {
 
         assertEquals(plaintext, out.toString());
         String verificationString = readStringFromFile(verifications);
-        assertEquals("2022-11-09T17:22:48Z C0DCEC44B1A173664B05DABCECD0BF863F65C9A5 A2EC077FC977E15DD799EFF92C0D3C123CF51C08\n",
+        assertEquals("2022-11-09T17:22:48Z C0DCEC44B1A173664B05DABCECD0BF863F65C9A5 A2EC077FC977E15DD799EFF92C0D3C123CF51C08 mode:binary\n",
                 verificationString);
     }
 
@@ -300,7 +304,8 @@ public class RoundTripEncryptDecryptCmdTest extends CLITest {
             InvalidAlgorithmParameterException, NoSuchAlgorithmException, IOException {
         PGPSecretKeyRing secretKeys = PGPainless.buildKeyRing()
                 .addUserId("No Crypt <no@crypt.key>")
-                .setPrimaryKey(KeySpec.getBuilder(KeyType.EDDSA(EdDSACurve._Ed25519), KeyFlag.CERTIFY_OTHER, KeyFlag.SIGN_DATA))
+                .setPrimaryKey(KeySpec.getBuilder(KeyType.EDDSA(EdDSACurve._Ed25519),
+                        KeyFlag.CERTIFY_OTHER, KeyFlag.SIGN_DATA))
                 .build();
         PGPPublicKeyRing cert = PGPainless.extractCertificate(secretKeys);
         File certFile = writeFile("cert.pgp", cert.getEncoded());
@@ -314,11 +319,13 @@ public class RoundTripEncryptDecryptCmdTest extends CLITest {
     }
 
     @Test
-    public void testSignWithIncapableKey() throws IOException, PGPException, InvalidAlgorithmParameterException, NoSuchAlgorithmException {
+    public void testSignWithIncapableKey()
+            throws IOException, PGPException, InvalidAlgorithmParameterException, NoSuchAlgorithmException {
         PGPSecretKeyRing secretKeys = PGPainless.buildKeyRing()
                 .addUserId("Cannot Sign <cannot@sign.key>")
                 .setPrimaryKey(KeySpec.getBuilder(KeyType.EDDSA(EdDSACurve._Ed25519), KeyFlag.CERTIFY_OTHER))
-                .addSubkey(KeySpec.getBuilder(KeyType.XDH(XDHSpec._X25519), KeyFlag.ENCRYPT_COMMS, KeyFlag.ENCRYPT_STORAGE))
+                .addSubkey(KeySpec.getBuilder(
+                        KeyType.XDH(XDHSpec._X25519), KeyFlag.ENCRYPT_COMMS, KeyFlag.ENCRYPT_STORAGE))
                 .build();
         File keyFile = writeFile("key.pgp", secretKeys.getEncoded());
         File certFile = writeFile("cert.pgp", PGPainless.extractCertificate(secretKeys).getEncoded());
@@ -558,4 +565,111 @@ public class RoundTripEncryptDecryptCmdTest extends CLITest {
                 keyFile.getAbsolutePath()));
         assertEquals(msg, out.toString());
     }
+
+    @Test
+    public void decryptMalformedMessageYieldsBadData() throws IOException {
+        // Message contains encrypted data packet which contains the plaintext directly - no literal data packet.
+        // It is therefore malformed.
+        String malformed = "-----BEGIN PGP MESSAGE-----\n" +
+                "Version: BCPG v1.72b04\n" +
+                "\n" +
+                "hF4D831k4umlLu4SAQdApKA6VDKSLQvwS2kbWqlhcXD8XHdFkSccqv5tBptZnBgw\n" +
+                "nZNXVhwUpap0ymb4jPTD+EVPKOfPyy04ouIGZAJKkfYDeSL/8sKcbnPPuQJYYjGQ\n" +
+                "ySDNmidrtTonwcSuwAfRyn74BBqOVhrr8GXkVIfevIlZFQ==\n" +
+                "=wIgl\n" +
+                "-----END PGP MESSAGE-----";
+        File key = writeFile("key.asc", KEY);
+        pipeStringToStdin(malformed);
+        int exitCode = executeCommand("decrypt", key.getAbsolutePath());
+        assertEquals(SOPGPException.BadData.EXIT_CODE, exitCode);
+    }
+
+    @Test
+    public void decryptWithPasswordWithPendingWhitespaceWorks() throws IOException {
+        assertEncryptWithPasswordADecryptWithPasswordBWorks("sw0rdf1sh", "sw0rdf1sh \n");
+    }
+
+    @Test
+    public void encryptWithTrailingWhitespaceDecryptWithoutWorks() throws IOException {
+        assertEncryptWithPasswordADecryptWithPasswordBWorks("sw0rdf1sh \n", "sw0rdf1sh");
+    }
+
+    @Test
+    public void decryptWithWhitespacePasswordWorks() throws IOException {
+        // is encrypted for "sw0rdf1sh \n"
+        String encryptedToPasswordWithTrailingWhitespace = "-----BEGIN PGP MESSAGE-----\n" +
+                "\n" +
+                "jA0ECQMC32tEJug0BCpg0kABfT3dKgA4K8XGpk2ul67BXLZD//fCCSmIQIWnNhE1\n" +
+                "6q97xFQ628K8f/58+XoBzLqLDT+LEz9Bz+Yg9QfzkEFy\n" +
+                "=2Y+K\n" +
+                "-----END PGP MESSAGE-----";
+        pipeStringToStdin(encryptedToPasswordWithTrailingWhitespace);
+        File password = writeFile("password", "sw0rdf1sh \n");
+        ByteArrayOutputStream plaintext = pipeStdoutToStream();
+        assertSuccess(executeCommand("decrypt", "--with-password", password.getAbsolutePath()));
+
+        assertEquals("Hello, World!\n", plaintext.toString());
+    }
+
+    private void assertEncryptWithPasswordADecryptWithPasswordBWorks(String passwordA, String passwordB)
+            throws IOException {
+        File passwordAFile = writeFile("password", passwordA);
+        File passwordBFile = writeFile("passwordWithWS", passwordB);
+
+        String msg = "Hello, World!\n";
+        pipeStringToStdin(msg);
+        ByteArrayOutputStream ciphertext = pipeStdoutToStream();
+        assertSuccess(executeCommand("encrypt", "--with-password", passwordAFile.getAbsolutePath()));
+
+        pipeStringToStdin(ciphertext.toString());
+        ByteArrayOutputStream plaintext = pipeStdoutToStream();
+        assertSuccess(executeCommand("decrypt", "--with-password", passwordBFile.getAbsolutePath()));
+
+        assertEquals(msg, plaintext.toString());
+    }
+
+    @Test
+    public void testDecryptWithoutDecryptionOptionFails() throws IOException {
+        String ciphertext = "-----BEGIN PGP MESSAGE-----\n" +
+                "Version: PGPainless\n" +
+                "\n" +
+                "hF4D831k4umlLu4SAQdAYisjZTDRm217LHQbqjB766tm62CKTkRj3Gd0wYxVRCgw\n" +
+                "48SnOJINCJoPgDsxk2NiJmLCImoiET7IElqxN9htdDXQJwcRK+71r/ZyO4YJpWuX\n" +
+                "0sAAAcEFc3nT+un31sOi8KoBJlc5n+MemntQvcWDs8B87BEW/Ncjrs0s4pJpZKBQ\n" +
+                "/AWc4wLCI3ylfMQJB2pICqaOO3KP3WepgTIw5fuZm6YfriKQi7uZvVx1N+uaCIoa\n" +
+                "K2IVVf/7O9KZJ9GbsGYdpBj9IdaIZiVS3Xi8rwgQl3haI/EeHC3nnCsWyj23Fjt3\n" +
+                "LjbMqpHbSnp8U1cQ8rXavrREaKv69PFeJio6/hRg32TzJqn05dPALRxHMEkxxa4h\n" +
+                "FpVU\n" +
+                "=edS5\n" +
+                "-----END PGP MESSAGE-----";
+        pipeStringToStdin(ciphertext);
+        int exitCode = executeCommand("decrypt");
+        assertEquals(SOPGPException.MissingArg.EXIT_CODE, exitCode);
+    }
+
+    @Test
+    public void testEncryptDecryptWithFreshRSAKey() throws IOException {
+        // Generate key
+        File passwordFile = writeFile("password", "sw0rdf1sh");
+        File keyFile = pipeStdoutToFile("key.asc");
+        assertSuccess(executeCommand("generate-key", "--profile=rfc4880", "--with-key-password", passwordFile.getAbsolutePath(), "Alice <alice@example.org>"));
+
+        File certFile = pipeStdoutToFile("cert.asc");
+        pipeFileToStdin(keyFile);
+        assertSuccess(executeCommand("extract-cert"));
+
+        // Write plaintext
+        File plaintextFile = writeFile("msg.txt", "Hello, World!\n");
+
+        // Encrypt
+        File ciphertextFile = pipeStdoutToFile("msg.asc");
+        pipeFileToStdin(plaintextFile);
+        assertSuccess(executeCommand("encrypt", "--profile=rfc4880", certFile.getAbsolutePath()));
+
+        ByteArrayOutputStream decrypted = pipeStdoutToStream();
+        pipeFileToStdin(ciphertextFile);
+        assertSuccess(executeCommand("decrypt", "--with-key-password", passwordFile.getAbsolutePath(), keyFile.getAbsolutePath()));
+
+        assertEquals("Hello, World!\n", decrypted.toString());
+    }
 }
