@@ -29,7 +29,7 @@ public class VersionCmdTest extends CLITest {
     public void testGetBackendVersion() throws IOException {
         ByteArrayOutputStream out = pipeStdoutToStream();
         assertSuccess(executeCommand("version", "--backend"));
-        assertTrue(out.toString().startsWith("Bouncy Castle "));
+        assertTrue(out.toString().startsWith("PGPainless "));
     }
 
     @Test
@@ -41,4 +41,12 @@ public class VersionCmdTest extends CLITest {
         assertTrue(info.contains("Bouncy Castle"));
         assertTrue(info.contains("Stateless OpenPGP Protocol"));
     }
+
+    @Test
+    public void testSopSpecVersion() throws IOException {
+        ByteArrayOutputStream out = pipeStdoutToStream();
+        assertSuccess(executeCommand("version", "--sop-spec"));
+        String info = out.toString();
+        assertTrue(info.startsWith("draft-dkg-openpgp-stateless-cli-"));
+    }
 }
