@@ -8,7 +8,10 @@ import java.io.BufferedOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.util.ArrayList;
+import java.util.HashSet;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Set;
 import javax.annotation.Nonnull;
 
 import org.bouncycastle.bcpg.ArmoredOutputStream;
@@ -22,11 +25,11 @@ import org.bouncycastle.openpgp.PGPSignatureGenerator;
 import org.bouncycastle.openpgp.operator.PGPDataEncryptorBuilder;
 import org.bouncycastle.openpgp.operator.PGPKeyEncryptionMethodGenerator;
 import org.pgpainless.algorithm.CompressionAlgorithm;
+import org.pgpainless.algorithm.HashAlgorithm;
 import org.pgpainless.algorithm.StreamEncoding;
 import org.pgpainless.algorithm.SymmetricKeyAlgorithm;
 import org.pgpainless.implementation.ImplementationFactory;
 import org.pgpainless.key.SubkeyIdentifier;
-import org.pgpainless.util.ArmorUtils;
 import org.pgpainless.util.ArmoredOutputStreamFactory;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -87,14 +90,6 @@ public final class EncryptionStream extends OutputStream {
 
         LOGGER.debug("Wrap encryption output in ASCII armor");
         armorOutputStream = ArmoredOutputStreamFactory.get(outermostStream, options);
-        if (options.hasComment()) {
-            String[] commentLines = options.getComment().split("\n");
-            for (String commentLine : commentLines) {
-                if (!commentLine.trim().isEmpty()) {
-                    ArmorUtils.addCommentHeader(armorOutputStream, commentLine.trim());
-                }
-            }
-        }
         outermostStream = armorOutputStream;
     }
 
@@ -165,9 +160,8 @@ public final class EncryptionStream extends OutputStream {
 
     private void prepareLiteralDataProcessing() throws IOException {
         if (options.isCleartextSigned()) {
-            // Begin cleartext with hash algorithm of first signing method
-            SigningOptions.SigningMethod firstMethod = options.getSigningOptions().getSigningMethods().values().iterator().next();
-            armorOutputStream.beginClearText(firstMethod.getHashAlgorithm().getAlgorithmId());
+            int[] algorithmIds = collectHashAlgorithmsForCleartextSigning();
+            armorOutputStream.beginClearText(algorithmIds);
             return;
         }
 
@@ -195,6 +189,24 @@ public final class EncryptionStream extends OutputStream {
         outermostStream = crlfGeneratorStream;
     }
 
+    private int[] collectHashAlgorithmsForCleartextSigning() {
+        SigningOptions signOpts = options.getSigningOptions();
+        Set<HashAlgorithm> hashAlgorithms = new HashSet<>();
+        if (signOpts != null) {
+            for (SigningOptions.SigningMethod method : signOpts.getSigningMethods().values()) {
+                hashAlgorithms.add(method.getHashAlgorithm());
+            }
+        }
+
+        int[] algorithmIds = new int[hashAlgorithms.size()];
+        Iterator<HashAlgorithm> iterator = hashAlgorithms.iterator();
+        for (int i = 0; i < algorithmIds.length; i++) {
+            algorithmIds[i] = iterator.next().getAlgorithmId();
+        }
+
+        return algorithmIds;
+    }
+
     @Override
     public void write(int data) throws IOException {
         outermostStream.write(data);
