@@ -6,7 +6,6 @@ package org.pgpainless.key.generation;
 
 
 import java.io.IOException;
-import java.nio.charset.Charset;
 import java.security.InvalidAlgorithmParameterException;
 import java.security.KeyPair;
 import java.security.KeyPairGenerator;
@@ -36,6 +35,7 @@ import org.bouncycastle.openpgp.operator.PBESecretKeyDecryptor;
 import org.bouncycastle.openpgp.operator.PBESecretKeyEncryptor;
 import org.bouncycastle.openpgp.operator.PGPContentSignerBuilder;
 import org.bouncycastle.openpgp.operator.PGPDigestCalculator;
+import org.bouncycastle.util.Strings;
 import org.pgpainless.PGPainless;
 import org.pgpainless.algorithm.HashAlgorithm;
 import org.pgpainless.algorithm.KeyFlag;
@@ -54,14 +54,13 @@ import org.pgpainless.util.Passphrase;
 
 public class KeyRingBuilder implements KeyRingBuilderInterface<KeyRingBuilder> {
 
-    @SuppressWarnings("CharsetObjectCanBeUsed")
-    private final Charset UTF8 = Charset.forName("UTF-8");
+    private static final long YEAR_IN_SECONDS = 1000L * 60 * 60 * 24 * 365;
 
     private KeySpec primaryKeySpec;
     private final List<KeySpec> subkeySpecs = new ArrayList<>();
     private final Map<String, SelfSignatureSubpackets.Callback> userIds = new LinkedHashMap<>();
     private Passphrase passphrase = Passphrase.emptyPassphrase();
-    private Date expirationDate = null;
+    private Date expirationDate = new Date(System.currentTimeMillis() + YEAR_IN_SECONDS * 5); // Expiration in 5 years
 
     @Override
     public KeyRingBuilder setPrimaryKey(@Nonnull KeySpec keySpec) {
@@ -93,11 +92,17 @@ public class KeyRingBuilder implements KeyRingBuilderInterface<KeyRingBuilder> {
 
     @Override
     public KeyRingBuilder addUserId(@Nonnull byte[] userId) {
-        return addUserId(new String(userId, UTF8));
+        return addUserId(Strings.fromUTF8ByteArray(userId));
     }
 
     @Override
-    public KeyRingBuilder setExpirationDate(@Nonnull Date expirationDate) {
+    public KeyRingBuilder setExpirationDate(@Nullable Date expirationDate) {
+        if (expirationDate == null) {
+            // No expiration
+            this.expirationDate = null;
+            return this;
+        }
+
         Date now = new Date();
         if (now.after(expirationDate)) {
             throw new IllegalArgumentException("Expiration date must be in the future.");
@@ -299,6 +304,16 @@ public class KeyRingBuilder implements KeyRingBuilderInterface<KeyRingBuilder> {
     public static PGPKeyPair generateKeyPair(KeySpec spec)
             throws NoSuchAlgorithmException, PGPException,
             InvalidAlgorithmParameterException {
+        Date keyCreationDate = spec.getKeyCreationDate();
+        if (keyCreationDate == null) {
+            keyCreationDate = new Date();
+        }
+        return generateKeyPair(spec, keyCreationDate);
+    }
+
+    public static PGPKeyPair generateKeyPair(KeySpec spec, Date keyCreationDate)
+            throws NoSuchAlgorithmException, PGPException,
+            InvalidAlgorithmParameterException {
         KeyType type = spec.getKeyType();
         KeyPairGenerator certKeyGenerator = KeyPairGenerator.getInstance(type.getName(),
                 ProviderFactory.getProvider());
@@ -307,8 +322,6 @@ public class KeyRingBuilder implements KeyRingBuilderInterface<KeyRingBuilder> {
         // Create raw Key Pair
         KeyPair keyPair = certKeyGenerator.generateKeyPair();
 
-        Date keyCreationDate = spec.getKeyCreationDate() != null ? spec.getKeyCreationDate() : new Date();
-
         // Form PGP key pair
         PGPKeyPair pgpKeyPair = ImplementationFactory.getInstance()
                 .getPGPKeyPair(type.getAlgorithm(), keyPair, keyCreationDate);
