@@ -4,17 +4,29 @@
 
 package org.pgpainless.key.modification.secretkeyring;
 
-import org.bouncycastle.bcpg.S2K;
-import org.bouncycastle.bcpg.SecretKeyPacket;
+import static org.pgpainless.key.util.KeyRingUtils.changePassphrase;
+import static org.pgpainless.util.CollectionUtils.concat;
+
+import java.io.IOException;
+import java.security.InvalidAlgorithmParameterException;
+import java.security.NoSuchAlgorithmException;
+import java.util.Collections;
+import java.util.Date;
+import java.util.List;
+import java.util.Map;
+import java.util.NoSuchElementException;
+import java.util.Set;
+import javax.annotation.Nonnull;
+import javax.annotation.Nullable;
+
 import org.bouncycastle.bcpg.sig.KeyExpirationTime;
 import org.bouncycastle.openpgp.PGPException;
 import org.bouncycastle.openpgp.PGPKeyPair;
 import org.bouncycastle.openpgp.PGPPublicKey;
+import org.bouncycastle.openpgp.PGPPublicKeyRing;
 import org.bouncycastle.openpgp.PGPSecretKey;
 import org.bouncycastle.openpgp.PGPSecretKeyRing;
 import org.bouncycastle.openpgp.PGPSignature;
-import org.bouncycastle.openpgp.operator.PBESecretKeyDecryptor;
-import org.bouncycastle.openpgp.operator.PBESecretKeyEncryptor;
 import org.pgpainless.PGPainless;
 import org.pgpainless.algorithm.AlgorithmSuite;
 import org.pgpainless.algorithm.CompressionAlgorithm;
@@ -34,8 +46,8 @@ import org.pgpainless.key.protection.KeyRingProtectionSettings;
 import org.pgpainless.key.protection.PasswordBasedSecretKeyRingProtector;
 import org.pgpainless.key.protection.SecretKeyRingProtector;
 import org.pgpainless.key.protection.UnprotectedKeysProtector;
-import org.pgpainless.key.protection.fixes.S2KUsageFix;
 import org.pgpainless.key.protection.passphrase_provider.SolitaryPassphraseProvider;
+import org.pgpainless.key.util.KeyIdUtil;
 import org.pgpainless.key.util.KeyRingUtils;
 import org.pgpainless.key.util.RevocationAttributes;
 import org.pgpainless.signature.builder.DirectKeySelfSignatureBuilder;
@@ -51,39 +63,27 @@ import org.pgpainless.signature.subpackets.SignatureSubpacketsUtil;
 import org.pgpainless.util.Passphrase;
 import org.pgpainless.util.selection.userid.SelectUserId;
 
-import javax.annotation.Nonnull;
-import javax.annotation.Nullable;
-import java.io.IOException;
-import java.security.InvalidAlgorithmParameterException;
-import java.security.NoSuchAlgorithmException;
-import java.util.ArrayList;
-import java.util.Collections;
-import java.util.Date;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.NoSuchElementException;
-import java.util.Set;
-
-import static org.pgpainless.util.CollectionUtils.concat;
-
 public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
 
     private PGPSecretKeyRing secretKeyRing;
     private final Date referenceTime;
 
-    public SecretKeyRingEditor(PGPSecretKeyRing secretKeyRing) {
-        this(secretKeyRing, null);
+    public SecretKeyRingEditor(@Nonnull PGPSecretKeyRing secretKeyRing) {
+        this(secretKeyRing, new Date());
     }
 
-    public SecretKeyRingEditor(PGPSecretKeyRing secretKeyRing, Date referenceTime) {
-        if (secretKeyRing == null) {
-            throw new NullPointerException("SecretKeyRing MUST NOT be null.");
-        }
+    public SecretKeyRingEditor(@Nonnull PGPSecretKeyRing secretKeyRing,
+                               @Nonnull Date referenceTime) {
         this.secretKeyRing = secretKeyRing;
         this.referenceTime = referenceTime;
     }
 
+    @Nonnull
+    @Override
+    public Date getReferenceTime() {
+        return referenceTime;
+    }
+
     @Override
     public SecretKeyRingEditorInterface addUserId(
             @Nonnull CharSequence userId,
@@ -126,9 +126,7 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
         }
 
         SelfSignatureBuilder builder = new SelfSignatureBuilder(primaryKey, protector);
-        if (referenceTime != null) {
-            builder.getHashedSubpackets().setSignatureCreationTime(referenceTime);
-        }
+        builder.getHashedSubpackets().setSignatureCreationTime(referenceTime);
         builder.setSignatureType(SignatureType.POSITIVE_CERTIFICATION);
 
         // Retain signature subpackets of previous signatures
@@ -158,7 +156,7 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
         PGPSignature signature = primaryUserId == null ?
                 info.getLatestDirectKeySelfSignature() : info.getLatestUserIdCertification(primaryUserId);
         final Date previousKeyExpiration = signature == null ? null :
-            SignatureSubpacketsUtil.getKeyExpirationTimeAsDate(signature, primaryKey);
+                SignatureSubpacketsUtil.getKeyExpirationTimeAsDate(signature, primaryKey);
 
         // Add new primary user-id signature
         addUserId(
@@ -184,7 +182,10 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
             }
 
             // We need to unmark this user-id as primary
-            if (info.getLatestUserIdCertification(otherUserId).getHashedSubPackets().isPrimaryUserID()) {
+            PGPSignature userIdCertification = info.getLatestUserIdCertification(otherUserId);
+            assert (userIdCertification != null);
+
+            if (userIdCertification.getHashedSubPackets().isPrimaryUserID()) {
                 addUserId(otherUserId, new SelfSignatureSubpackets.Callback() {
                     @Override
                     public void modifyHashedSubpackets(SelfSignatureSubpackets hashedSubpackets) {
@@ -290,6 +291,7 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
             @Override
             public void modifyHashedSubpackets(SelfSignatureSubpackets hashedSubpackets) {
                 SignatureSubpacketsHelper.applyFrom(keySpec.getSubpackets(), (SignatureSubpackets) hashedSubpackets);
+                hashedSubpackets.setSignatureCreationTime(referenceTime);
             }
         };
 
@@ -307,7 +309,7 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
             @Nullable SelfSignatureSubpackets.Callback subpacketsCallback,
             @Nonnull SecretKeyRingProtector secretKeyRingProtector)
             throws PGPException, InvalidAlgorithmParameterException, NoSuchAlgorithmException, IOException {
-        PGPKeyPair keyPair = KeyRingBuilder.generateKeyPair(keySpec);
+        PGPKeyPair keyPair = KeyRingBuilder.generateKeyPair(keySpec, referenceTime);
 
         SecretKeyRingProtector subKeyProtector = PasswordBasedSecretKeyRingProtector
                 .forKeyId(keyPair.getKeyID(), subkeyPassphrase);
@@ -327,7 +329,7 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
             @Nonnull SecretKeyRingProtector primaryKeyProtector,
             @Nonnull KeyFlag keyFlag,
             KeyFlag... additionalKeyFlags)
-            throws PGPException, IOException, NoSuchAlgorithmException {
+            throws PGPException, IOException {
         KeyFlag[] flags = concat(keyFlag, additionalKeyFlags);
         PublicKeyAlgorithm subkeyAlgorithm = PublicKeyAlgorithm.requireFromId(subkey.getPublicKey().getAlgorithm());
         SignatureSubpacketsUtil.assureKeyCanCarryFlags(subkeyAlgorithm);
@@ -350,16 +352,12 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
                 .getV4FingerprintCalculator(), false, subkeyProtector.getEncryptor(subkey.getKeyID()));
 
         SubkeyBindingSignatureBuilder skBindingBuilder = new SubkeyBindingSignatureBuilder(primaryKey, primaryKeyProtector, hashAlgorithm);
-        if (referenceTime != null) {
-            skBindingBuilder.getHashedSubpackets().setSignatureCreationTime(referenceTime);
-        }
+        skBindingBuilder.getHashedSubpackets().setSignatureCreationTime(referenceTime);
         skBindingBuilder.getHashedSubpackets().setKeyFlags(flags);
 
         if (subkeyAlgorithm.isSigningCapable()) {
             PrimaryKeyBindingSignatureBuilder pkBindingBuilder = new PrimaryKeyBindingSignatureBuilder(secretSubkey, subkeyProtector, hashAlgorithm);
-            if (referenceTime != null) {
-                pkBindingBuilder.getHashedSubpackets().setSignatureCreationTime(referenceTime);
-            }
+            pkBindingBuilder.getHashedSubpackets().setSignatureCreationTime(referenceTime);
             PGPSignature pkBinding = pkBindingBuilder.build(primaryKey.getPublicKey());
             skBindingBuilder.getHashedSubpackets().addEmbeddedSignature(pkBinding);
         }
@@ -412,8 +410,8 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
     }
 
     @Override
-    public PGPSignature createRevocationCertificate(@Nonnull SecretKeyRingProtector secretKeyRingProtector,
-                                                    @Nullable RevocationAttributes revocationAttributes)
+    public PGPSignature createRevocation(@Nonnull SecretKeyRingProtector secretKeyRingProtector,
+                                         @Nullable RevocationAttributes revocationAttributes)
             throws PGPException {
         PGPPublicKey revokeeSubKey = secretKeyRing.getPublicKey();
         PGPSignature revocationCertificate = generateRevocation(
@@ -422,7 +420,7 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
     }
 
     @Override
-    public PGPSignature createRevocationCertificate(
+    public PGPSignature createRevocation(
             long subkeyId,
             @Nonnull SecretKeyRingProtector secretKeyRingProtector,
             @Nullable RevocationAttributes revocationAttributes)
@@ -433,7 +431,7 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
     }
 
     @Override
-    public PGPSignature createRevocationCertificate(
+    public PGPSignature createRevocation(
             long subkeyId,
             @Nonnull SecretKeyRingProtector secretKeyRingProtector,
             @Nullable RevocationSignatureSubpackets.Callback certificateSubpacketsCallback)
@@ -603,6 +601,7 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
             }
 
             if (prevUserIdSig.getHashedSubPackets().isPrimaryUserID()) {
+                assert (primaryUserId != null);
                 PGPSignature userIdSig = reissueNonPrimaryUserId(secretKeyRingProtector, userId, prevUserIdSig);
                 secretKeyRing = KeyRingUtils.injectCertification(secretKeyRing, primaryUserId, userIdSig);
             }
@@ -611,6 +610,42 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
         return this;
     }
 
+    @Override
+    public SecretKeyRingEditorInterface setExpirationDateOfSubkey(@Nullable Date expiration, long keyId, @Nonnull SecretKeyRingProtector secretKeyRingProtector) throws PGPException {
+        // is primary key
+        if (keyId == secretKeyRing.getPublicKey().getKeyID()) {
+            return setExpirationDate(expiration, secretKeyRingProtector);
+        }
+
+        // is subkey
+        PGPPublicKey subkey = secretKeyRing.getPublicKey(keyId);
+        if (subkey != null) {
+            PGPSignature prevBinding = PGPainless.inspectKeyRing(secretKeyRing).getCurrentSubkeyBindingSignature(keyId);
+            PGPSignature bindingSig = reissueSubkeyBindingSignature(subkey, expiration, secretKeyRingProtector, prevBinding);
+            secretKeyRing = KeyRingUtils.injectCertification(secretKeyRing, subkey, bindingSig);
+        } else {
+            throw new NoSuchElementException("No subkey with ID " + KeyIdUtil.formatKeyId(keyId) + " found.");
+        }
+        return this;
+    }
+
+    @Override
+    public PGPPublicKeyRing createMinimalRevocationCertificate(
+            @Nonnull SecretKeyRingProtector secretKeyRingProtector,
+            @Nullable RevocationAttributes keyRevocationAttributes)
+            throws PGPException {
+        // Check reason
+        if (keyRevocationAttributes != null && !RevocationAttributes.Reason.isKeyRevocation(keyRevocationAttributes.getReason())) {
+            throw new IllegalArgumentException("Revocation reason MUST be applicable to a key revocation.");
+        }
+
+        PGPSignature revocation = createRevocation(secretKeyRingProtector, keyRevocationAttributes);
+        PGPPublicKey primaryKey = secretKeyRing.getSecretKey().getPublicKey();
+        primaryKey = KeyRingUtils.getStrippedDownPublicKey(primaryKey);
+        primaryKey = PGPPublicKey.addCertification(primaryKey, revocation);
+        return new PGPPublicKeyRing(Collections.singletonList(primaryKey));
+    }
+
     private PGPSignature reissueNonPrimaryUserId(
             SecretKeyRingProtector secretKeyRingProtector,
             String userId,
@@ -684,6 +719,57 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
         return builder.build(publicKey);
     }
 
+    private PGPSignature reissueSubkeyBindingSignature(
+            PGPPublicKey subkey,
+            Date expiration,
+            SecretKeyRingProtector secretKeyRingProtector,
+            PGPSignature prevSubkeyBindingSignature)
+        throws PGPException {
+        if (subkey == null) {
+            throw new IllegalArgumentException("Subkey MUST NOT be null.");
+        }
+        if (prevSubkeyBindingSignature == null) {
+            throw new IllegalArgumentException("Previous subkey binding signature for "
+                    + KeyIdUtil.formatKeyId(subkey.getKeyID()) + " MUST NOT be null.");
+        }
+        PGPPublicKey primaryKey = secretKeyRing.getPublicKey();
+        PGPSecretKey secretPrimaryKey = secretKeyRing.getSecretKey();
+        PGPSecretKey secretSubkey = secretKeyRing.getSecretKey(subkey.getKeyID());
+
+        if (secretPrimaryKey == null) {
+            throw new NoSuchElementException("Secret Key Ring does not contain secret-key component for the primary key.");
+        }
+
+        SubkeyBindingSignatureBuilder builder = new SubkeyBindingSignatureBuilder(
+                secretPrimaryKey, secretKeyRingProtector, prevSubkeyBindingSignature);
+        SelfSignatureSubpackets subpackets = builder.getHashedSubpackets();
+        if (referenceTime != null) {
+            subpackets.setSignatureCreationTime(referenceTime);
+        }
+        // Set expiration
+        subpackets.setKeyExpirationTime(subkey, expiration);
+        subpackets.setSignatureExpirationTime(null); // avoid copying sig exp time
+
+        List<KeyFlag> previousKeyFlags = SignatureSubpacketsUtil.parseKeyFlags(prevSubkeyBindingSignature);
+        if (previousKeyFlags != null && previousKeyFlags.contains(KeyFlag.SIGN_DATA)) {
+            if (secretSubkey == null) {
+                throw new NoSuchElementException("Secret keyring does not contain secret-key component for subkey " +
+                        KeyIdUtil.formatKeyId(subkey.getKeyID()));
+            }
+
+            // Create new embedded back-sig
+            subpackets.clearEmbeddedSignatures();
+            try {
+                subpackets.addEmbeddedSignature(
+                        new PrimaryKeyBindingSignatureBuilder(secretSubkey, secretKeyRingProtector)
+                                .build(primaryKey));
+            } catch (IOException e) {
+                throw new PGPException("Cannot add embedded primary-key back signature.", e);
+            }
+        }
+        return builder.build(subkey);
+    }
+
     private PGPSignature getPreviousDirectKeySignature() {
         KeyRingInfo info = PGPainless.inspectKeyRing(secretKeyRing, referenceTime);
         return info.getLatestDirectKeySelfSignature();
@@ -793,67 +879,4 @@ public class SecretKeyRingEditor implements SecretKeyRingEditorInterface {
         }
     }
 
-    private PGPSecretKeyRing changePassphrase(Long keyId,
-                                              PGPSecretKeyRing secretKeys,
-                                              SecretKeyRingProtector oldProtector,
-                                              SecretKeyRingProtector newProtector)
-            throws PGPException {
-        List<PGPSecretKey> secretKeyList = new ArrayList<>();
-        if (keyId == null) {
-            // change passphrase of whole key ring
-            Iterator<PGPSecretKey> secretKeyIterator = secretKeys.getSecretKeys();
-            while (secretKeyIterator.hasNext()) {
-                PGPSecretKey secretKey = secretKeyIterator.next();
-                secretKey = reencryptPrivateKey(secretKey, oldProtector, newProtector);
-                secretKeyList.add(secretKey);
-            }
-        } else {
-            // change passphrase of selected subkey only
-            Iterator<PGPSecretKey> secretKeyIterator = secretKeys.getSecretKeys();
-            while (secretKeyIterator.hasNext()) {
-                PGPSecretKey secretKey = secretKeyIterator.next();
-                if (secretKey.getPublicKey().getKeyID() == keyId) {
-                    // Re-encrypt only the selected subkey
-                    secretKey = reencryptPrivateKey(secretKey, oldProtector, newProtector);
-                }
-                secretKeyList.add(secretKey);
-            }
-        }
-
-        PGPSecretKeyRing newRing = new PGPSecretKeyRing(secretKeyList);
-        newRing = s2kUsageFixIfNecessary(newRing, newProtector);
-        return newRing;
-    }
-
-    private PGPSecretKeyRing s2kUsageFixIfNecessary(PGPSecretKeyRing secretKeys, SecretKeyRingProtector protector)
-            throws PGPException {
-        boolean hasS2KUsageChecksum = false;
-        for (PGPSecretKey secKey : secretKeys) {
-            if (secKey.getS2KUsage() == SecretKeyPacket.USAGE_CHECKSUM) {
-                hasS2KUsageChecksum = true;
-                break;
-            }
-        }
-        if (hasS2KUsageChecksum) {
-            secretKeys = S2KUsageFix.replaceUsageChecksumWithUsageSha1(
-                    secretKeys, protector, true);
-        }
-        return secretKeys;
-    }
-
-    private static PGPSecretKey reencryptPrivateKey(
-            PGPSecretKey secretKey,
-            SecretKeyRingProtector oldProtector,
-            SecretKeyRingProtector newProtector)
-            throws PGPException {
-        S2K s2k = secretKey.getS2K();
-        // If the key uses GNU_DUMMY_S2K, we leave it as is and skip this block
-        if (s2k == null || s2k.getType() != S2K.GNU_DUMMY_S2K) {
-            long secretKeyId = secretKey.getKeyID();
-            PBESecretKeyDecryptor decryptor = oldProtector.getDecryptor(secretKeyId);
-            PBESecretKeyEncryptor encryptor = newProtector.getEncryptor(secretKeyId);
-            secretKey = PGPSecretKey.copyWithNewPassword(secretKey, decryptor, encryptor);
-        }
-        return secretKey;
-    }
 }
