@@ -13,6 +13,7 @@ import javax.annotation.Nullable;
 
 import org.bouncycastle.openpgp.PGPException;
 import org.bouncycastle.openpgp.PGPKeyPair;
+import org.bouncycastle.openpgp.PGPPublicKeyRing;
 import org.bouncycastle.openpgp.PGPSecretKeyRing;
 import org.bouncycastle.openpgp.PGPSignature;
 import org.pgpainless.algorithm.KeyFlag;
@@ -28,6 +29,14 @@ import org.pgpainless.util.selection.userid.SelectUserId;
 
 public interface SecretKeyRingEditorInterface {
 
+    /**
+     * Return the editors reference time.
+     *
+     * @return reference time
+     */
+    @Nonnull
+    Date getReferenceTime();
+
     /**
      * Add a user-id to the key ring.
      *
@@ -179,7 +188,6 @@ public interface SecretKeyRingEditorInterface {
      *
      * @throws PGPException in case we cannot generate a binding signature for the subkey
      * @throws IOException in case of an IO error
-     * @throws NoSuchAlgorithmException in case of missing algorithm support in the crypto backend
      */
     SecretKeyRingEditorInterface addSubKey(
             @Nonnull PGPKeyPair subkey,
@@ -188,7 +196,7 @@ public interface SecretKeyRingEditorInterface {
             @Nonnull SecretKeyRingProtector primaryKeyProtector,
             @Nonnull KeyFlag keyFlag,
             KeyFlag... additionalKeyFlags)
-            throws PGPException, IOException, NoSuchAlgorithmException;
+            throws PGPException, IOException;
 
     /**
      * Revoke the key ring.
@@ -461,8 +469,42 @@ public interface SecretKeyRingEditorInterface {
             @Nonnull SecretKeyRingProtector secretKeyRingProtector)
             throws PGPException;
 
+    /**
+     * Set the expiration date for the subkey identified by the given keyId to the given expiration date.
+     * If the key is supposed to never expire, then an expiration date of null is expected.
+     *
+     * @param expiration new expiration date of null
+     * @param keyId id of the subkey
+     * @param secretKeyRingProtector to unlock the secret key
+     * @return the builder
+     * @throws PGPException in case we cannot generate a new subkey-binding or self-signature with the
+     *                      changed expiration date
+     */
+    SecretKeyRingEditorInterface setExpirationDateOfSubkey(
+            @Nullable Date expiration,
+            long keyId,
+            @Nonnull SecretKeyRingProtector secretKeyRingProtector)
+        throws PGPException;
+
+    /**
+     * Create a minimal, self-authorizing revocation certificate, containing only the primary key
+     * and a revocation signature.
+     * This type of revocation certificates was introduced in OpenPGP v6.
+     * This method has no side effects on the original key and will leave it intact.
+     *
+     * @param secretKeyRingProtector protector to unlock the primary key.
+     * @param keyRevocationAttributes reason for the revocation (key revocation)
+     * @return minimal revocation certificate
+     *
+     * @throws PGPException in case we cannot generate a revocation signature
+     */
+    PGPPublicKeyRing createMinimalRevocationCertificate(@Nonnull SecretKeyRingProtector secretKeyRingProtector,
+                                                        @Nullable RevocationAttributes keyRevocationAttributes)
+    throws PGPException;
+
     /**
      * Create a detached revocation certificate, which can be used to revoke the whole key.
+     * The original key will not be modified by this method.
      *
      * @param secretKeyRingProtector protector to unlock the primary key.
      * @param revocationAttributes reason for the revocation
@@ -470,13 +512,14 @@ public interface SecretKeyRingEditorInterface {
      *
      * @throws PGPException in case we cannot generate a revocation certificate
      */
-    PGPSignature createRevocationCertificate(
+    PGPSignature createRevocation(
             @Nonnull SecretKeyRingProtector secretKeyRingProtector,
             @Nullable RevocationAttributes revocationAttributes)
             throws PGPException;
 
     /**
      * Create a detached revocation certificate, which can be used to revoke the specified subkey.
+     * The original key will not be modified by this method.
      *
      * @param subkeyId id of the subkey to be revoked
      * @param secretKeyRingProtector protector to unlock the primary key.
@@ -485,7 +528,7 @@ public interface SecretKeyRingEditorInterface {
      *
      * @throws PGPException in case we cannot generate a revocation certificate
      */
-    PGPSignature createRevocationCertificate(
+    PGPSignature createRevocation(
             long subkeyId,
             @Nonnull SecretKeyRingProtector secretKeyRingProtector,
             @Nullable RevocationAttributes revocationAttributes)
@@ -493,6 +536,7 @@ public interface SecretKeyRingEditorInterface {
 
     /**
      * Create a detached revocation certificate, which can be used to revoke the specified subkey.
+     * The original key will not be modified by this method.
      *
      * @param subkeyId id of the subkey to be revoked
      * @param secretKeyRingProtector protector to unlock the primary key.
@@ -501,7 +545,7 @@ public interface SecretKeyRingEditorInterface {
      *
      * @throws PGPException in case we cannot generate a revocation certificate
      */
-    PGPSignature createRevocationCertificate(
+    PGPSignature createRevocation(
             long subkeyId,
             @Nonnull SecretKeyRingProtector secretKeyRingProtector,
             @Nullable RevocationSignatureSubpackets.Callback certificateSubpacketsCallback)
@@ -509,6 +553,7 @@ public interface SecretKeyRingEditorInterface {
 
     /**
      * Create a detached revocation certificate, which can be used to revoke the specified subkey.
+     * The original key will not be modified by this method.
      *
      * @param subkeyFingerprint fingerprint of the subkey to be revoked
      * @param secretKeyRingProtector protector to unlock the primary key.
@@ -517,13 +562,13 @@ public interface SecretKeyRingEditorInterface {
      *
      * @throws PGPException in case we cannot generate a revocation certificate
      */
-    default PGPSignature createRevocationCertificate(
+    default PGPSignature createRevocation(
             OpenPgpFingerprint subkeyFingerprint,
             SecretKeyRingProtector secretKeyRingProtector,
             @Nullable RevocationAttributes revocationAttributes)
             throws PGPException {
 
-        return createRevocationCertificate(
+        return createRevocation(
                 subkeyFingerprint.getKeyId(),
                 secretKeyRingProtector,
                 revocationAttributes);
