@@ -14,6 +14,8 @@ import java.util.NoSuchElementException;
 import javax.annotation.Nonnull;
 import javax.annotation.Nullable;
 
+import org.bouncycastle.bcpg.S2K;
+import org.bouncycastle.bcpg.SecretKeyPacket;
 import org.bouncycastle.openpgp.PGPException;
 import org.bouncycastle.openpgp.PGPKeyRing;
 import org.bouncycastle.openpgp.PGPPrivateKey;
@@ -25,10 +27,16 @@ import org.bouncycastle.openpgp.PGPSecretKeyRing;
 import org.bouncycastle.openpgp.PGPSecretKeyRingCollection;
 import org.bouncycastle.openpgp.PGPSignature;
 import org.bouncycastle.openpgp.PGPUserAttributeSubpacketVector;
+import org.bouncycastle.openpgp.operator.PBESecretKeyDecryptor;
+import org.bouncycastle.openpgp.operator.PBESecretKeyEncryptor;
+import org.bouncycastle.util.Strings;
 import org.pgpainless.PGPainless;
 import org.pgpainless.implementation.ImplementationFactory;
 import org.pgpainless.key.protection.SecretKeyRingProtector;
 import org.pgpainless.key.protection.UnlockSecretKey;
+import org.pgpainless.key.protection.fixes.S2KUsageFix;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 public final class KeyRingUtils {
 
@@ -36,6 +44,8 @@ public final class KeyRingUtils {
 
     }
 
+    private static final Logger LOGGER = LoggerFactory.getLogger(KeyRingUtils.class);
+
     /**
      * Return the primary {@link PGPSecretKey} from the provided {@link PGPSecretKeyRing}.
      * If it has no primary secret key, throw a {@link NoSuchElementException}.
@@ -144,6 +154,17 @@ public final class KeyRingUtils {
         return secretKey;
     }
 
+    @Nonnull
+    public static PGPPublicKeyRing publicKeys(@Nonnull PGPKeyRing keys) {
+        if (keys instanceof PGPPublicKeyRing) {
+            return (PGPPublicKeyRing) keys;
+        } else if (keys instanceof PGPSecretKeyRing) {
+            return publicKeyRingFrom((PGPSecretKeyRing) keys);
+        } else {
+            throw new IllegalArgumentException("Unknown keys class: " + keys.getClass().getName());
+        }
+    }
+
     /**
      * Extract a {@link PGPPublicKeyRing} containing all public keys from the provided {@link PGPSecretKeyRing}.
      *
@@ -167,12 +188,9 @@ public final class KeyRingUtils {
      *
      * @param secretKeyRings secret key ring collection
      * @return public key ring collection
-     * @throws PGPException TODO: remove
-     * @throws IOException TODO: remove
      */
     @Nonnull
-    public static PGPPublicKeyRingCollection publicKeyRingCollectionFrom(@Nonnull PGPSecretKeyRingCollection secretKeyRings)
-            throws PGPException, IOException {
+    public static PGPPublicKeyRingCollection publicKeyRingCollectionFrom(@Nonnull PGPSecretKeyRingCollection secretKeyRings) {
         List<PGPPublicKeyRing> certificates = new ArrayList<>();
         for (PGPSecretKeyRing secretKey : secretKeyRings) {
             certificates.add(PGPainless.extractCertificate(secretKey));
@@ -200,13 +218,9 @@ public final class KeyRingUtils {
      *
      * @param rings array of public key rings
      * @return key ring collection
-     *
-     * @throws IOException in case of an io error
-     * @throws PGPException in case of a broken key
      */
     @Nonnull
-    public static PGPPublicKeyRingCollection keyRingsToKeyRingCollection(@Nonnull PGPPublicKeyRing... rings)
-            throws IOException, PGPException {
+    public static PGPPublicKeyRingCollection keyRingsToKeyRingCollection(@Nonnull PGPPublicKeyRing... rings) {
         return new PGPPublicKeyRingCollection(Arrays.asList(rings));
     }
 
@@ -215,13 +229,9 @@ public final class KeyRingUtils {
      *
      * @param rings array of secret key rings
      * @return secret key ring collection
-     *
-     * @throws IOException in case of an io error
-     * @throws PGPException in case of a broken key
      */
     @Nonnull
-    public static PGPSecretKeyRingCollection keyRingsToKeyRingCollection(@Nonnull PGPSecretKeyRing... rings)
-            throws IOException, PGPException {
+    public static PGPSecretKeyRingCollection keyRingsToKeyRingCollection(@Nonnull PGPSecretKeyRing... rings) {
         return new PGPSecretKeyRingCollection(Arrays.asList(rings));
     }
 
@@ -440,30 +450,6 @@ public final class KeyRingUtils {
         return newSecretKey;
     }
 
-    /**
-     * Remove the secret key of the subkey identified by the given secret key id from the key ring.
-     * The public part stays attached to the key ring, so that it can still be used for encryption / verification of signatures.
-     *
-     * This method is intended to be used to remove secret primary keys from live keys when those are kept in offline storage.
-     *
-     * @param secretKeys secret key ring
-     * @param secretKeyId id of the secret key to remove
-     * @return secret key ring with removed secret key
-     *
-     * @throws IOException in case of an error during serialization / deserialization of the key
-     * @throws PGPException in case of a broken key
-     *
-     * @deprecated use {@link #stripSecretKey(PGPSecretKeyRing, long)} instead.
-     * TODO: Remove in 1.2.X
-     */
-    @Nonnull
-    @Deprecated
-    public static PGPSecretKeyRing removeSecretKey(@Nonnull PGPSecretKeyRing secretKeys,
-                                                   long secretKeyId)
-            throws IOException, PGPException {
-        return stripSecretKey(secretKeys, secretKeyId);
-    }
-
     /**
      * Remove the secret key of the subkey identified by the given secret key id from the key ring.
      * The public part stays attached to the key ring, so that it can still be used for encryption / verification of signatures.
@@ -509,4 +495,95 @@ public final class KeyRingUtils {
         // Parse the key back into an object
         return new PGPSecretKeyRing(encoded.toByteArray(), ImplementationFactory.getInstance().getKeyFingerprintCalculator());
     }
+
+    /**
+     * Strip all user-ids, user-attributes and signatures from the given public key.
+     *
+     * @param bloatedKey public key
+     * @return stripped public key
+     * @throws PGPException if the packet is faulty or the required calculations fail
+     */
+    public static PGPPublicKey getStrippedDownPublicKey(PGPPublicKey bloatedKey) throws PGPException {
+        return new PGPPublicKey(bloatedKey.getPublicKeyPacket(), ImplementationFactory.getInstance().getKeyFingerprintCalculator());
+    }
+
+    public static List<String> getUserIdsIgnoringInvalidUTF8(PGPPublicKey key) {
+        List<String> userIds = new ArrayList<>();
+        Iterator<byte[]> it = key.getRawUserIDs();
+        while (it.hasNext()) {
+            byte[] rawUserId = it.next();
+            try {
+                userIds.add(Strings.fromUTF8ByteArray(rawUserId));
+            } catch (IllegalArgumentException e) {
+                LOGGER.warn("Invalid UTF-8 user-ID encountered: " + new String(rawUserId));
+            }
+        }
+        return userIds;
+    }
+
+    public static PGPSecretKeyRing changePassphrase(Long keyId,
+                                              PGPSecretKeyRing secretKeys,
+                                              SecretKeyRingProtector oldProtector,
+                                              SecretKeyRingProtector newProtector)
+            throws PGPException {
+        List<PGPSecretKey> secretKeyList = new ArrayList<>();
+        if (keyId == null) {
+            // change passphrase of whole key ring
+            Iterator<PGPSecretKey> secretKeyIterator = secretKeys.getSecretKeys();
+            while (secretKeyIterator.hasNext()) {
+                PGPSecretKey secretKey = secretKeyIterator.next();
+                secretKey = KeyRingUtils.reencryptPrivateKey(secretKey, oldProtector, newProtector);
+                secretKeyList.add(secretKey);
+            }
+        } else {
+            // change passphrase of selected subkey only
+            Iterator<PGPSecretKey> secretKeyIterator = secretKeys.getSecretKeys();
+            while (secretKeyIterator.hasNext()) {
+                PGPSecretKey secretKey = secretKeyIterator.next();
+                if (secretKey.getPublicKey().getKeyID() == keyId) {
+                    // Re-encrypt only the selected subkey
+                    secretKey = KeyRingUtils.reencryptPrivateKey(secretKey, oldProtector, newProtector);
+                }
+                secretKeyList.add(secretKey);
+            }
+        }
+
+        PGPSecretKeyRing newRing = new PGPSecretKeyRing(secretKeyList);
+        newRing = s2kUsageFixIfNecessary(newRing, newProtector);
+        return newRing;
+    }
+
+
+    public static PGPSecretKey reencryptPrivateKey(
+            PGPSecretKey secretKey,
+            SecretKeyRingProtector oldProtector,
+            SecretKeyRingProtector newProtector)
+            throws PGPException {
+        S2K s2k = secretKey.getS2K();
+        // If the key uses GNU_DUMMY_S2K, we leave it as is and skip this block
+        if (s2k == null || s2k.getType() != S2K.GNU_DUMMY_S2K) {
+            long secretKeyId = secretKey.getKeyID();
+            PBESecretKeyDecryptor decryptor = oldProtector.getDecryptor(secretKeyId);
+            PBESecretKeyEncryptor encryptor = newProtector.getEncryptor(secretKeyId);
+            secretKey = PGPSecretKey.copyWithNewPassword(secretKey, decryptor, encryptor);
+        }
+        return secretKey;
+    }
+
+
+    public static PGPSecretKeyRing s2kUsageFixIfNecessary(PGPSecretKeyRing secretKeys, SecretKeyRingProtector protector)
+            throws PGPException {
+        boolean hasS2KUsageChecksum = false;
+        for (PGPSecretKey secKey : secretKeys) {
+            if (secKey.getS2KUsage() == SecretKeyPacket.USAGE_CHECKSUM) {
+                hasS2KUsageChecksum = true;
+                break;
+            }
+        }
+        if (hasS2KUsageChecksum) {
+            secretKeys = S2KUsageFix.replaceUsageChecksumWithUsageSha1(
+                    secretKeys, protector, true);
+        }
+        return secretKeys;
+    }
 }
