@@ -9,12 +9,18 @@ import java.io.OutputStream;
 import org.bouncycastle.bcpg.ArmoredOutputStream;
 import org.pgpainless.encryption_signing.ProducerOptions;
 
+import javax.annotation.Nonnull;
+import javax.annotation.Nullable;
+
 /**
  * Factory to create configured {@link ArmoredOutputStream ArmoredOutputStreams}.
  * The configuration entails setting custom version and comment headers.
  */
 public final class ArmoredOutputStreamFactory {
 
+    /**
+     * Name of the program.
+     */
     public static final String PGPAINLESS = "PGPainless";
     private static String version = PGPAINLESS;
     private static String[] comment = new String[0];
@@ -23,33 +29,49 @@ public final class ArmoredOutputStreamFactory {
 
     }
 
+    private static ArmoredOutputStream.Builder getBuilder() {
+        ArmoredOutputStream.Builder builder = ArmoredOutputStream.builder();
+        builder.clearHeaders();
+        if (version != null && !version.isEmpty()) {
+            builder.setVersion(version);
+        }
+        for (String comment : comment) {
+            builder.addComment(comment);
+        }
+        return builder;
+    }
+
     /**
      * Wrap an {@link OutputStream} inside a preconfigured {@link ArmoredOutputStream}.
      *
      * @param outputStream inner stream
      * @return armored output stream
      */
-    public static ArmoredOutputStream get(OutputStream outputStream) {
-        ArmoredOutputStream armoredOutputStream = new ArmoredOutputStream(outputStream);
-        armoredOutputStream.clearHeaders();
-        if (version != null && !version.isEmpty()) {
-            armoredOutputStream.setHeader(ArmorUtils.HEADER_VERSION, version);
-        }
-
-        for (String comment : comment) {
-            ArmorUtils.addCommentHeader(armoredOutputStream, comment);
-        }
-        return armoredOutputStream;
+    @Nonnull
+    public static ArmoredOutputStream get(@Nonnull OutputStream outputStream) {
+        return getBuilder().build(outputStream);
     }
 
-    public static ArmoredOutputStream get(OutputStream outputStream, ProducerOptions options) {
+    /**
+     * Return an instance of the {@link ArmoredOutputStream} which might have pre-populated armor headers.
+     *
+     * @param outputStream output stream
+     * @param options options
+     * @return armored output stream
+     */
+    @Nonnull
+    public static ArmoredOutputStream get(@Nonnull OutputStream outputStream, @Nonnull ProducerOptions options) {
+        ArmoredOutputStream.Builder builder = getBuilder();
         if (options.isHideArmorHeaders()) {
-            ArmoredOutputStream armorOut = new ArmoredOutputStream(outputStream);
-            armorOut.clearHeaders();
-            return armorOut;
-        } else {
-            return get(outputStream);
+            builder.clearHeaders();
         }
+        if (options.hasVersion()) {
+            builder.setVersion(options.getVersion());
+        }
+        if (options.hasComment()) {
+            builder.setComment(options.getComment());
+        }
+        return builder.build(outputStream);
     }
 
     /**
@@ -59,7 +81,7 @@ public final class ArmoredOutputStreamFactory {
      *
      * @param versionString version string
      */
-    public static void setVersionInfo(String versionString) {
+    public static void setVersionInfo(@Nullable String versionString) {
         if (versionString == null) {
             version = null;
             return;
@@ -88,7 +110,7 @@ public final class ArmoredOutputStreamFactory {
      *
      * @param commentString comment
      */
-    public static void setComment(String commentString) {
+    public static void setComment(@Nullable String commentString) {
         if (commentString == null) {
             throw new IllegalArgumentException("Comment cannot be null.");
         }
