@@ -4,6 +4,7 @@
 
 package org.pgpainless.decryption_verification;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 
 import java.io.ByteArrayInputStream;
@@ -238,8 +239,10 @@ public class ModificationDetectionTests {
                 );
 
         ByteArrayOutputStream out = new ByteArrayOutputStream();
-        Streams.pipeAll(decryptionStream, out);
-        assertThrows(ModificationDetectionException.class, decryptionStream::close);
+        assertThrows(ModificationDetectionException.class, () -> {
+            Streams.pipeAll(decryptionStream, out);
+            decryptionStream.close();
+        });
     }
 
     @TestTemplate
@@ -269,8 +272,10 @@ public class ModificationDetectionTests {
                 );
 
         ByteArrayOutputStream out = new ByteArrayOutputStream();
-        Streams.pipeAll(decryptionStream, out);
-        assertThrows(ModificationDetectionException.class, decryptionStream::close);
+        assertThrows(ModificationDetectionException.class, () -> {
+            Streams.pipeAll(decryptionStream, out);
+            decryptionStream.close();
+        });
     }
 
     @TestTemplate
@@ -313,8 +318,10 @@ public class ModificationDetectionTests {
                 );
 
         ByteArrayOutputStream out = new ByteArrayOutputStream();
-        Streams.pipeAll(decryptionStream, out);
-        assertThrows(ModificationDetectionException.class, decryptionStream::close);
+        assertThrows(ModificationDetectionException.class, () -> {
+            Streams.pipeAll(decryptionStream, out);
+            decryptionStream.close();
+        });
     }
 
     @TestTemplate
@@ -344,8 +351,10 @@ public class ModificationDetectionTests {
                 );
 
         ByteArrayOutputStream out = new ByteArrayOutputStream();
-        Streams.pipeAll(decryptionStream, out);
-        assertThrows(ModificationDetectionException.class, decryptionStream::close);
+        assertThrows(ModificationDetectionException.class, () -> {
+            Streams.pipeAll(decryptionStream, out);
+            decryptionStream.close();
+        });
     }
 
     @TestTemplate
@@ -366,7 +375,7 @@ public class ModificationDetectionTests {
 
     @TestTemplate
     @ExtendWith(TestAllImplementations.class)
-    public void decryptMessageWithSEDPacket() throws IOException {
+    public void decryptMessageWithSEDPacket() throws IOException, PGPException {
         Passphrase passphrase = Passphrase.fromPassword("flowcrypt compatibility tests");
         String key = "-----BEGIN PGP PRIVATE KEY BLOCK-----\r\n" +
                 "Version: FlowCrypt 6.9.1 Gmail Encryption\r\n" +
@@ -528,9 +537,24 @@ public class ModificationDetectionTests {
                 .withOptions(new ConsumerOptions().addDecryptionKey(secretKeyRing,
                         SecretKeyRingProtector.unlockAnyKeyWith(passphrase)))
         );
+
+        DecryptionStream decryptionStream = PGPainless.decryptAndOrVerify()
+                .onInputStream(new ByteArrayInputStream(ciphertext.getBytes(StandardCharsets.UTF_8)))
+                .withOptions(ConsumerOptions.get().addDecryptionKey(secretKeyRing,
+                        SecretKeyRingProtector.unlockAnyKeyWith(passphrase))
+                        .setIgnoreMDCErrors(true));
+        ByteArrayOutputStream plaintext = new ByteArrayOutputStream();
+        Streams.pipeAll(decryptionStream, plaintext);
+        decryptionStream.close();
+
+        assertEquals("As stated in subject\r\n" +
+                "\r\n" +
+                "Shall not decrypt automatically\r\n" +
+                "\r\n" +
+                "Has to show a warning\r\n", plaintext.toString());
     }
 
-    private PGPSecretKeyRingCollection getDecryptionKey() throws IOException, PGPException {
+    private PGPSecretKeyRingCollection getDecryptionKey() throws IOException {
         PGPSecretKeyRing secretKeys = PGPainless.readKeyRing().secretKeyRing(keyAscii);
         return new PGPSecretKeyRingCollection(Collections.singletonList(secretKeys));
     }
