@@ -29,6 +29,9 @@ import sop.enums.InlineSignAs;
 import sop.exception.SOPGPException;
 import sop.operation.InlineSign;
 
+/**
+ * Implementation of the <pre>inline-sign</pre> operation using PGPainless.
+ */
 public class InlineSignImpl implements InlineSign {
 
     private boolean armor = true;
@@ -71,7 +74,7 @@ public class InlineSignImpl implements InlineSign {
     }
 
     @Override
-    public Ready data(InputStream data) throws SOPGPException.KeyIsProtected, IOException, SOPGPException.ExpectedText {
+    public Ready data(InputStream data) throws SOPGPException.KeyIsProtected, SOPGPException.ExpectedText {
         for (PGPSecretKeyRing key : signingKeys) {
             try {
                 if (mode == InlineSignAs.clearsigned) {
