@@ -7,6 +7,7 @@ package org.pgpainless.sop;
 import org.pgpainless.util.ArmoredOutputStreamFactory;
 import sop.SOP;
 import sop.operation.Armor;
+import sop.operation.ChangeKeyPassword;
 import sop.operation.Dearmor;
 import sop.operation.Decrypt;
 import sop.operation.DetachedSign;
@@ -17,8 +18,16 @@ import sop.operation.ExtractCert;
 import sop.operation.GenerateKey;
 import sop.operation.InlineSign;
 import sop.operation.InlineVerify;
+import sop.operation.ListProfiles;
+import sop.operation.RevokeKey;
 import sop.operation.Version;
 
+/**
+ * Implementation of the <pre>sop</pre> API using PGPainless.
+ * <pre> {@code
+ * SOP sop = new SOPImpl();
+ * }</pre>
+ */
 public class SOPImpl implements SOP {
 
     static {
@@ -90,6 +99,21 @@ public class SOPImpl implements SOP {
         return new DearmorImpl();
     }
 
+    @Override
+    public ListProfiles listProfiles() {
+        return new ListProfilesImpl();
+    }
+
+    @Override
+    public RevokeKey revokeKey() {
+        return new RevokeKeyImpl();
+    }
+
+    @Override
+    public ChangeKeyPassword changeKeyPassword() {
+        return new ChangeKeyPasswordImpl();
+    }
+
     @Override
     public InlineDetach inlineDetach() {
         return new InlineDetachImpl();
