@@ -21,7 +21,9 @@ import sop.ByteArrayAndResult;
 import sop.DecryptionResult;
 import sop.SOP;
 import sop.SessionKey;
+import sop.enums.SignatureMode;
 import sop.exception.SOPGPException;
+import sop.testsuite.assertions.VerificationListAssert;
 
 public class EncryptDecryptRoundTripTest {
 
@@ -75,7 +77,8 @@ public class EncryptDecryptRoundTripTest {
         assertArrayEquals(message, decrypted.toByteArray());
 
         DecryptionResult result = bytesAndResult.getResult();
-        assertEquals(1, result.getVerifications().size());
+        VerificationListAssert.assertThatVerificationList(result.getVerifications())
+                .hasSingleItem();
     }
 
     @Test
@@ -106,7 +109,8 @@ public class EncryptDecryptRoundTripTest {
         assertArrayEquals(message, decrypted);
 
         DecryptionResult result = bytesAndResult.getResult();
-        assertEquals(1, result.getVerifications().size());
+        VerificationListAssert.assertThatVerificationList(result.getVerifications())
+                .hasSingleItem();
     }
 
     @Test
@@ -125,7 +129,8 @@ public class EncryptDecryptRoundTripTest {
         assertArrayEquals(message, decrypted);
 
         DecryptionResult result = bytesAndResult.getResult();
-        assertEquals(0, result.getVerifications().size());
+        VerificationListAssert.assertThatVerificationList(result.getVerifications())
+                .isEmpty();
     }
 
     @Test
@@ -144,7 +149,8 @@ public class EncryptDecryptRoundTripTest {
         assertArrayEquals(message, decrypted);
 
         DecryptionResult result = bytesAndResult.getResult();
-        assertEquals(0, result.getVerifications().size());
+        VerificationListAssert.assertThatVerificationList(result.getVerifications())
+                .isEmpty();
     }
 
     @Test
@@ -163,7 +169,8 @@ public class EncryptDecryptRoundTripTest {
         assertArrayEquals(message, decrypted);
 
         DecryptionResult result = bytesAndResult.getResult();
-        assertEquals(0, result.getVerifications().size());
+        VerificationListAssert.assertThatVerificationList(result.getVerifications())
+                .isEmpty();
     }
 
     @Test
@@ -180,7 +187,8 @@ public class EncryptDecryptRoundTripTest {
                 .toByteArrayAndResult()
                 .getResult();
 
-        assertTrue(result.getVerifications().isEmpty());
+        VerificationListAssert.assertThatVerificationList(result.getVerifications())
+                .isEmpty();
     }
 
     @Test
@@ -486,14 +494,19 @@ public class EncryptDecryptRoundTripTest {
                 sop.decrypt().withKey(key).verifyWithCert(cert).ciphertext(ciphertext).toByteArrayAndResult();
         assertEquals(sessionKey, bytesAndResult.getResult().getSessionKey().get().toString());
         assertArrayEquals(plaintext, bytesAndResult.getBytes());
-        assertEquals(1, bytesAndResult.getResult().getVerifications().size());
-
+        VerificationListAssert.assertThatVerificationList(bytesAndResult.getResult().getVerifications())
+                .hasSingleItem()
+                .issuedBy("9C26EFAB1C6500A228E8A9C2658EE420C824D191")
+                .hasMode(SignatureMode.binary);
         // Decrypt with session key
         bytesAndResult = sop.decrypt().withSessionKey(SessionKey.fromString(sessionKey))
                 .verifyWithCert(cert).ciphertext(ciphertext).toByteArrayAndResult();
         assertEquals(sessionKey, bytesAndResult.getResult().getSessionKey().get().toString());
         assertArrayEquals(plaintext, bytesAndResult.getBytes());
-        assertEquals(1, bytesAndResult.getResult().getVerifications().size());
+        VerificationListAssert.assertThatVerificationList(bytesAndResult.getResult().getVerifications())
+                .hasSingleItem()
+                .issuedBy("9C26EFAB1C6500A228E8A9C2658EE420C824D191")
+                .hasMode(SignatureMode.binary);
     }
 
     @Test
@@ -512,7 +525,7 @@ public class EncryptDecryptRoundTripTest {
                 "-----END PGP MESSAGE-----").getBytes(StandardCharsets.UTF_8);
         SessionKey wrongSessionKey = SessionKey.fromString("9:63F741E7FB60247BE59C64158573308F727236482DB7653908C95839E4166AAE");
 
-        assertThrows(SOPGPException.BadData.class, () ->
+        assertThrows(SOPGPException.CannotDecrypt.class, () ->
                 sop.decrypt().withSessionKey(wrongSessionKey).ciphertext(ciphertext));
     }
 
@@ -533,4 +546,43 @@ public class EncryptDecryptRoundTripTest {
                         .toByteArrayAndResult()
         );
     }
+
+    @Test
+    public void encryptWithUnsupportedProfileFails() {
+        assertThrows(SOPGPException.UnsupportedProfile.class, () -> sop
+                .encrypt()
+                .profile("Unknown"));
+    }
+
+    @Test
+    public void encryptWithSupportedProfileTest() throws IOException {
+
+        byte[] key = sop.generateKey()
+                .profile("rfc4880")
+                .userId("Alice <alice@pgpainless.org>")
+                .generate()
+                .getBytes();
+
+        byte[] cert = sop.extractCert()
+                .key(key)
+                .getBytes();
+
+        byte[] encrypted = sop.encrypt()
+                .profile("rfc4880")
+                .withCert(cert)
+                .plaintext(message)
+                .getBytes();
+
+        ByteArrayAndResult<DecryptionResult> bytesAndResult = sop.decrypt()
+                .withKey(key)
+                .ciphertext(encrypted)
+                .toByteArrayAndResult();
+
+        ByteArrayOutputStream decrypted = new ByteArrayOutputStream();
+        Streams.pipeAll(bytesAndResult.getInputStream(), decrypted);
+        assertArrayEquals(message, decrypted.toByteArray());
+
+        DecryptionResult result = bytesAndResult.getResult();
+        assertTrue(result.getSessionKey().isPresent());
+    }
 }
